@@ -305,9 +305,8 @@ int main(int argc, char* argv[])
 
         if(!S_ISDIR(stat_.st_mode)) {
             mcurses_init(code);
-            saphire_set_signal();
+            saphire_set_signal_optc();
             saphire_init(kATOptC, FALSE, runtime_script, FALSE);
-            //saphire_init(kATConsoleApp, FALSE, runtime_script, FALSE);
 
             saphire_set_global_var("SCRIPT_FILE_NAME", file_name);
 
@@ -349,12 +348,13 @@ int main(int argc, char* argv[])
     /// オプションで指定されているならシェルで実行する ///
     if(send_cmd[0] == 'c') {
         mcurses_init(code);
-        saphire_set_signal();
+        saphire_set_signal_optc();
         saphire_init(kATOptC, FALSE, runtime_script, FALSE);
         sWFd* pipeout = sWFd_new(STDOUT_FILENO);
         sRFd* pipein = sRFd_new(STDIN_FILENO);
         int rcode = saphire_shell(send_cmd + 1, "saphire -a", pipeout, pipein, STDERR_FILENO);
         saphire_sweep();
+
         sWFd_flash(pipeout);
         sWFd_delete(pipeout);
         sRFd_delete(pipein);
