@@ -36,6 +36,388 @@
 #define OnigUChar unsigned char
 #endif
 
+void saphire_get_quoted_fname_readline_ver(char* file, string_obj* ret);
+
+typedef struct {
+    string_obj* mName;
+
+    struct stat mStat;                // stat()のキャッシュ
+} sFile;
+
+sFile* sFile_new(char* name, struct stat* stat_)
+{
+    sFile* self = MALLOC(sizeof(sFile));
+
+    self->mName = STRING_NEW(name);
+    self->mStat = *stat_;
+
+    return self;
+}
+
+void sFile_delete(sFile* self)
+{
+    string_delete(self->mName);
+    FREE(self);
+}
+
+static BOOL sort_name(void* left, void* right)
+{
+    sFile* l = (sFile*) left;
+    sFile* r = (sFile*) right;
+
+    char* lfname = string_c_str(l->mName);
+    char* rfname = string_c_str(r->mName);
+
+    if(strcmp(lfname, ".") == 0) return TRUE;
+    if(strcmp(lfname, "..") == 0) {
+        if(strcmp(rfname, ".") == 0) return FALSE;
+
+        return TRUE;          
+    }
+    if(strcmp(rfname, ".") == 0) return FALSE;
+    if(strcmp(rfname, "..") == 0) return FALSE;
+
+    if(!S_ISDIR(l->mStat.st_mode) && !S_ISDIR(r->mStat.st_mode)
+        || S_ISDIR(l->mStat.st_mode) && S_ISDIR(r->mStat.st_mode) )
+    {
+        return strcasecmp(lfname, rfname) < 0;
+    }
+
+    if(S_ISDIR(l->mStat.st_mode))
+        return TRUE;
+    else
+        return FALSE;
+}
+
+// pathの最後は/をつける
+static BOOL readline_select_file_read(vector_obj* v, char* path)
+{
+    DIR* dir = opendir(path);
+    if(dir == NULL) {
+        return FALSE;
+    }
+    
+    struct dirent* entry;
+    while(entry = readdir(dir)) {
+        char fpath[PATH_MAX];
+        strcpy(fpath, path);
+        strcat(fpath, entry->d_name);
+
+        struct stat stat_;
+        memset(&stat_, 0, sizeof(struct stat));
+        if(stat(fpath, &stat_) < 0) {
+            continue;
+        }
+
+        sFile* file = sFile_new(entry->d_name, &stat_);
+
+        vector_add(v, file);
+    }
+    
+    closedir(dir);
+
+    /// ソート ///
+    vector_sort(v, sort_name);
+
+    return TRUE;
+}
+
+
+static int readline_select_file(int count, int key)
+{
+    msave_ttysettings();       // 端末の設定の保存
+    msave_screen();
+    minitscr();
+
+    const int maxx = mgetmaxx();
+    const int maxy = mgetmaxy();
+
+    vector_obj* v = VECTOR_NEW(10);
+    char cwd_path[PATH_MAX];
+    mygetcwd(cwd_path);
+    strcat(cwd_path, "/");
+
+    char current_dir[PATH_MAX];
+    strcpy(current_dir, cwd_path);
+    readline_select_file_read(v, current_dir);
+
+    vector_obj* mark_files = VECTOR_NEW(10);
+
+    if(vector_size(v) > 0) {
+        int cursor = 0;
+        while(1) {
+            /// 描写 ///
+            mclear();
+            mmvprintw(0,0,"%s", current_dir);
+
+            int page = cursor/((maxy-1)*4);
+            int n = page*4*(maxy-1);
+            while(n < page*4*(maxy-1)+(maxy-1)*4 && n < vector_size(v)) {
+                sFile* file = vector_item(v, n);
+
+                int attrs = 0;
+                if(n == cursor) {
+                    attrs |= kCAReverse;
+                }
+                if(S_ISDIR(file->mStat.st_mode)) {
+                    attrs |= kCACyan;
+                    attrs |= kCABold;
+                }
+                if(attrs) mattron(attrs);
+
+                char file2[PATH_MAX];
+                str_cut2(kUtf8, string_c_str(file->mName)
+                        , maxx/4-2, file2, PATH_MAX);
+
+                int y = (n-page*4*(maxy-1)) / 4 + 1;
+                int x = ((n-page*4*(maxy-1)) % 4)*maxx/4;
+
+                char full_path[PATH_MAX];
+                strcpy(full_path, current_dir);
+                strcat(full_path, string_c_str(file->mName));
+
+                BOOL marked = FALSE;
+                int i;
+                for(i=0; i<vector_size(mark_files); i++) {
+                    string_obj* mark_file = vector_item(mark_files, i);
+
+                    if(strcmp(string_c_str(mark_file), full_path) == 0) {
+                        marked = TRUE;
+                        break;
+                    }
+                }
+
+                if(marked) 
+                    mmvprintw(y, x, "*%s", file2);
+                else
+                    mmvprintw(y, x, " %s", file2);
+
+                if(attrs) {
+                    mattroff();
+                }
+
+                n++;
+            }
+            mrefresh();
+
+            /// インプット ///
+            int meta;
+            int key = mgetch(&meta);
+
+            if(key == 14 || key == KEY_DOWN) {
+                cursor+=4;
+            }
+            else if(key == 16 || key == KEY_UP) {
+                cursor-=4;
+            }
+            else if(key == 6 || key == KEY_RIGHT) {
+                cursor++;
+            }
+            else if(key == 2 || key == KEY_LEFT) {
+                cursor--;
+            }
+            else if(key == 4 || key == KEY_NPAGE) {
+                cursor += (maxy-1)*4;
+            }
+            else if(key == 21 || key == KEY_PPAGE) {
+                cursor -= (maxy-1)*4;
+            }
+            else if(key == 12) {
+                mclear();
+                mrefresh();
+            }
+            else if(key == ' ') {
+                sFile* cursor_file = vector_item(v, cursor);
+
+                string_obj* cursor_mfile = NULL;
+
+                char cursor_path[PATH_MAX];
+                strcpy(cursor_path, current_dir);
+                strcat(cursor_path, string_c_str(cursor_file->mName));
+
+                int i;
+                for(i=0; i<vector_size(mark_files); i++) {
+                    string_obj* mfile = vector_item(mark_files, i);
+
+                    if(strcmp(string_c_str(mfile), cursor_path) == 0) {
+                        cursor_mfile = mfile;
+                        break;
+                    }
+                }
+
+                if(!cursor_mfile) {
+                    vector_add(mark_files, STRING_NEW(cursor_path));
+                }
+                else {
+                    vector_erase(mark_files
+                        , vector_index(mark_files, cursor_mfile));
+                    string_delete(cursor_mfile);
+                }
+
+                cursor++;
+            }
+            else if(key == '\\') {
+                strcpy(current_dir, "/");
+
+                int i;
+                for(i=0; i<vector_size(v); i++) {
+                    sFile_delete(vector_item(v, i));
+                }
+                vector_clear(v);
+
+                readline_select_file_read(v, current_dir);
+                cursor = 0;
+            }
+            else if(key == '\t' || key == 'w') {
+                if(vector_size(mark_files) == 0) {
+                    sFile* cursor_file = vector_item(v, cursor);
+
+                    /// 現在のカレントディレクトリのパスなら
+                    /// ディレクトリ名を取る
+                    char cursor_path[PATH_MAX];
+                    strcpy(cursor_path, current_dir);
+                    strcat(cursor_path, string_c_str(cursor_file->mName));
+
+                    /// カレントのファイル、子ディレクトリのファイルなら
+                    /// 相対ディレクトリ
+                    if(memcmp(cursor_path
+                            , cwd_path, strlen(cwd_path)) == 0 )
+                    {
+                        char cursor_path2[PATH_MAX];
+                        memcpy(cursor_path2
+                            , cursor_path+strlen(cwd_path)
+                            , strlen(cursor_path) - strlen(cwd_path)+1);
+
+                        string_obj* quoted_fname = STRING_NEW("");
+                        saphire_get_quoted_fname_readline_ver(cursor_path2, quoted_fname);
+                        rl_insert_text(string_c_str(quoted_fname));
+                        string_delete(quoted_fname);
+                    }
+                    else {
+                        string_obj* quoted_fname = STRING_NEW("");
+                        saphire_get_quoted_fname_readline_ver(cursor_path, quoted_fname);
+                        rl_insert_text(string_c_str(quoted_fname));
+                        string_delete(quoted_fname);
+                    }
+                }
+                else {
+                    int j;
+                    for(j=0; j<vector_size(mark_files); j++) {
+                        char full_path[PATH_MAX];
+                        strcpy(full_path
+                            , string_c_str(vector_item(mark_files, j)));
+
+                        if(strstr(full_path, cwd_path) == full_path) {
+                            char full_path2[PATH_MAX];
+                            memcpy(full_path2
+                                , full_path + strlen(cwd_path) 
+                                , strlen(full_path) - strlen(cwd_path));
+                            full_path2[strlen(full_path) - strlen(cwd_path)] = 0;
+
+                            string_obj* quoted_fname = STRING_NEW("");
+                            saphire_get_quoted_fname_readline_ver
+                                (full_path2, quoted_fname);
+                            rl_insert_text(string_c_str(quoted_fname));
+                            string_delete(quoted_fname);
+
+                            if(j<vector_size(mark_files)-1) 
+                                rl_insert_text(" ");
+                        }
+                        else {
+                            string_obj* quoted_fname = STRING_NEW("");
+                            saphire_get_quoted_fname_readline_ver(full_path, quoted_fname);
+                            rl_insert_text(string_c_str(quoted_fname));
+                            string_delete(quoted_fname);
+
+                            if(j<vector_size(mark_files)-1) 
+                                rl_insert_text(" ");
+                        }
+                    }
+                }
+                break;
+            }
+            else if(key == 'q' || key == 3 | key == 27 || key == 7) {
+                break;
+            }
+            else if(key == 10 || key == 13) {
+                sFile* cursor_file = vector_item(v, cursor);
+
+                if(strcmp(string_c_str(cursor_file->mName), ".") != 0
+                    && S_ISDIR(cursor_file->mStat.st_mode)) 
+                {
+                    char path2[PATH_MAX];
+                    if(correct_path(current_dir
+                            , string_c_str(cursor_file->mName), path2)) 
+                    {
+                        strcpy(current_dir, path2);
+                        if(current_dir[strlen(current_dir)-1] != '/') {
+                            strcat(current_dir, "/");
+                        }
+
+                        int i;
+                        for(i=0; i<vector_size(v); i++) {
+                            sFile_delete(vector_item(v, i));
+                        }
+                        vector_clear(v);
+
+                        readline_select_file_read(v, current_dir);
+                        cursor = 0;
+                    }
+                }
+            }
+            else if(key == 8 || key == KEY_BACKSPACE) {    // CTRL-H
+                char parent_path[PATH_MAX];
+                parentname(parent_path, current_dir);
+                strcpy(current_dir, parent_path);
+
+                int i;
+                for(i=0; i<vector_size(v); i++) {
+                    sFile_delete(vector_item(v, i));
+                }
+                vector_clear(v);
+
+                readline_select_file_read(v, current_dir);
+                cursor = 0;
+            }
+
+            /// 修正 ///
+            if(cursor < 0) {
+                cursor = 0;
+            }
+            if(cursor >= vector_size(v)) {
+                cursor = vector_size(v)-1;
+            }
+        }
+    }
+
+    int i;
+    for(i=0; i<vector_size(v); i++) {
+        sFile_delete(vector_item(v, i));
+    }
+    vector_delete(v);
+
+    for(i=0; i<vector_size(mark_files); i++) {
+        string_delete(vector_item(mark_files, i));
+    }
+    vector_delete(mark_files);
+
+//mclear();
+//mrefresh();
+    mendwin();
+    mrestore_screen();
+    mrestore_ttysettings();    // 端末の設定の復帰
+
+    return 0;
+}
+
+
+
+
+
+
+
+
+
 //////////////////////////////////////////////////////////////////////
 // static variable
 //////////////////////////////////////////////////////////////////////
@@ -673,8 +1055,8 @@ char* readline_program_completion(const char* text, int stat)
         index = 0;
     }
 
-    while(index < vector_size(gPrograms)) {
-        char* program = string_c_str(vector_item(gPrograms, index));
+    while(index < vector_size(gSaphireProgCompletions)) {
+        char* program = string_c_str(vector_item(gSaphireProgCompletions, index));
         index++;
 
         if(!strncmp(text, program, wordlen)) {
@@ -685,159 +1067,46 @@ char* readline_program_completion(const char* text, int stat)
     return NULL;
 }
 
-static vector_obj* gCompList;               // 環境変数補完候補
-
-static void readline_env_completion_make_completion_list()
-{
-    int i;
-    for(i=0; i<vector_size(gCompList); i++) {
-        string_delete(vector_item(gCompList, i));
-    }
-    vector_clear(gCompList);
-
-    /// 環境変数 ///
-#if !HAVE_DECL_ENVIRON
-    extern char **environ;
-
-    char** p;
-    for(p = environ; *p; p++) {
-        char env_name[256];
-
-        char* p2 = env_name;
-        char* p3 = *p;
-
-        *p2++ = '$';
-        while(*p3 != '=') {
-            *p2++ = *p3++;
-        }
-        *p2 = 0;
-
-        vector_add(gCompList, STRING_NEW(env_name));
-    }
-#endif
-
-    hash_it* it = hash_loop_begin(gGlobals);
-    while(it) {
-        char* key = hash_loop_key(it);
-
-        char env_name[256];
-        strcpy(env_name, "$");
-        strcat(env_name, key);
-
-        vector_add(gCompList, STRING_NEW(env_name));
-
-        it = hash_loop_next(it);
-    }
-
-    it = hash_loop_begin(gArrays);
-    while(it) {
-        char* key = hash_loop_key(it);
-
-        char env_name[256];
-        strcpy(env_name, "$");
-        strcat(env_name, key);
-
-        vector_add(gCompList, STRING_NEW(env_name));
-
-        it = hash_loop_next(it);
-    }
-
-    it = hash_loop_begin(gHashs);
-    while(it) {
-        char* key = hash_loop_key(it);
-
-        char env_name[256];
-        strcpy(env_name, "$");
-        strcat(env_name, key);
-
-        vector_add(gCompList, STRING_NEW(env_name));
-
-        it = hash_loop_next(it);
-    }
-
-    (void)vector_sort(gCompList, name_sort);
-}
-
 char* readline_env_completion(const char* text, int stat)
 {
     static int index, wordlen;
-    static hash_it* git;
-    static hash_it* ait;
-    static hash_it* hit;
 
     if(stat == 0) {
         wordlen = strlen(text);
         index = 0;
     
-        readline_env_completion_make_completion_list();
+        saphire_make_var_completion_list();
     }
     while(1) {
-        if(index < vector_size(gCompList)) {
-            char* env = string_c_str(vector_item(gCompList, index));
+        if(index < vector_size(gSaphireVarCompletions)) {
+            char* env = string_c_str(vector_item(gSaphireVarCompletions, index));
+            char* env2 = malloc(strlen(env) + 10);
+
+            if(*text == '$' && *(text+1) != '$') {
+                strcpy(env2, "$");
+                strcat(env2, env);
+            }
+            else if(*text == '$' && *(text+1) == '$') {
+                strcpy(env2, "$$");
+                strcat(env2, env);
+            }
+            else if(*text == '@' && *(text+1) != '@') {
+                strcpy(env2, "@");
+                strcat(env2, env);
+            }
+            else if(*text == '@' && *(text+1) == '@') {
+                strcpy(env2, "@@");
+                strcat(env2, env);
+            }
+
             index++;
-            if(strncmp(text, env, wordlen) == 0) {
-                return strdup(env);
+
+            if(strncmp(text, env2, wordlen) == 0) {
+                return env2;
             }
+
+            free(env2);
         }
-/*
-        else if(git != NULL) {
-            char* global = hash_loop_key(git);
-            git = hash_loop_next(git);
-
-            if(!strncmp(text2, global, wordlen2)) {
-                char* str = malloc(strlen(global) + 1 +dollar_num +atmark_num);
-                char * p = str;
-                int i;
-                for(i=0; i<dollar_num; i++) {
-                    *p++ = '$';
-                }
-                for(i=0; i<atmark_num; i++) {
-                    *p++ = '@';
-                }
-                strcpy(p, global);
-
-                return str;
-            }
-        }
-        else if(ait != NULL) {
-            char* array = hash_loop_key(ait);
-            ait = hash_loop_next(ait);
-
-            if(!strncmp(text2, array, wordlen2)) {
-                char* str = malloc(strlen(array) + 1 +dollar_num +atmark_num);
-                char * p = str;
-                int i;
-                for(i=0; i<dollar_num; i++) {
-                    *p++ = '$';
-                }
-                for(i=0; i<atmark_num; i++) {
-                    *p++ = '@';
-                }
-                strcpy(p, array);
-
-                return str;
-            }
-        }
-        else if(hit != NULL) {
-            char* hash = hash_loop_key(hit);
-            hit = hash_loop_next(hit);
-
-            if(!strncmp(text2, hash, wordlen2)) {
-                char* str = malloc(strlen(hash) + 1 +dollar_num +atmark_num);
-                char * p = str;
-                int i;
-                for(i=0; i<dollar_num; i++) {
-                    *p++ = '$';
-                }
-                for(i=0; i<atmark_num; i++) {
-                    *p++ = '@';
-                }
-                strcpy(p, hash);
-
-                return str;
-            }
-        }
-*/
         else {
             break;
         }
@@ -940,10 +1209,23 @@ char** readline_on_complete(const char* text, int start, int end)
         }
         // 環境変数 $
         else if(!squote && *p == '$') {
+            p++;
+            env = TRUE;
+            user = FALSE;
+        }
+
+        // 環境変数 @
+        else if(!squote && *p == '@' && *(p+1) == '@') {
             p+=2;
             env = TRUE;
             user = FALSE;
         }
+        // 環境変数 @@
+        else if(!squote && *p == '@') {
+            p++;
+            env = TRUE;
+            user = FALSE;
+        }
         /// シングルクォート、ダブルクォート中 ///
         else if(squote || dquote) {
             p++;
@@ -1778,18 +2060,15 @@ void readline_init()
     migemo_init();
 #endif
 
-    gCompList = VECTOR_NEW(100);
     gEditingDir = STRING_NEW("");
+
+    rl_bind_key('x'-'a'+1, readline_select_file);
 }
 
 void readline_final()
 {
     string_delete(gEditingDir);
     int i;
-    for(i=0; i<vector_size(gCompList); i++) {
-        string_delete(vector_item(gCompList, i));
-    }
-    vector_delete(gCompList);
     if(files) {
         int i;
         for(i=0; i<vector_size(files); i++) {
