@@ -172,7 +172,7 @@ int correct_path(char* current_path, char* path, char* path2)
 
     /// 先頭にカレントパスを加える ///
     {
-        if(path[0] == '/' || path[0] == '~' || path[0] == '$') {
+        if(path[0] == '/') {
             strcpy(tmp, path);
         }
         else {
@@ -195,57 +195,8 @@ int correct_path(char* current_path, char* path, char* path2)
 
     }
 
-    /// ~と環境変数を展開する ///
-    {
-        char* home = getenv("HOME");
+    strcpy(tmp2, tmp);
 
-        if(home == NULL) {
-            fprintf(stderr, "$HOME is not setted");
-            exit(1);
-        }
-        
-        if(tmp[0] == '~' && strlen(tmp) == 1) {
-            strcpy(tmp2, home);
-        }
-        else if(tmp[0] == '~' && tmp[1] == '/') {
-            strcpy(tmp2, home);
-            strcat(tmp2, "/");
-            strcat(tmp2, tmp + 2);
-        }
-        else if(tmp[0] == '$') {
-            char* p;
-            char* p2;
-            char env[256];
-            char* env_value;
-
-            p = tmp + 1;
-            p2 = env;
-            while(*p != '/' && *p) {
-                *p2++ = *p++;
-            }
-            *p2 = 0;
-
-            env_value = getenv(env);
-            if(env_value == NULL) {
-                strcpy(tmp2, p);
-            }
-            else {
-                if(*p == '/') {
-                    strcpy(tmp2, env_value);
-                    strcat(tmp2, p);
-                }
-                else {
-                    strcpy(tmp2, env_value);
-                }
-            }
-        }
-        else {
-            strcpy(tmp2, tmp);
-        }
-
-    }
-
-   
     /// .を削除する ///
     {
         char* p;
