@@ -55,6 +55,7 @@ enum eCommandKind {
     kEach,
     kForeach,
 //    kParse,
+    kInnerCommand,
     kCommand 
 };
 
@@ -91,11 +92,6 @@ extern list_obj* gSigChldRecieved;
 extern char* gMagicNumber;
     // ֥ȥեΥޥåʥС
 
-extern hash_obj* gFileHandles;
-    //  ե̾
-    //  եǥץ
-    // ֥ȥեΥޥåʥС
-
 extern hash_obj* gReadBuffers;
     //  ե̾
     //  sRFd*
@@ -112,6 +108,8 @@ extern string_obj* gGlobalPipe;
 
 extern string_obj* gGlobalPipeNum[10];
     // ʥСХѥפɤ߹ߤǡ
+extern BOOL gSaphireBackground;
+    // SaphireХå饦ɤưƤ뤫ɤ
 
 extern hash_obj* gRegexs;
     // ɽΥå
@@ -185,7 +183,6 @@ BOOL xclose(int fd);
     // closeȤˤΥեǥץ
     // gFReadBuffersΥХåեк
     // gFWriteBuffersΥХåեн񤭹Ǻ
-    // gFileHandles˥եϥɥ뤬к
 
 void update_ary_env(char* key);
     // ѿˤޤĤѿ(_size)ͤ˹
