@@ -26,7 +26,7 @@ char gTmpDir[PATH_MAX];
 
 string_obj* gErrMsg;
     // エラーメッセージ
-vector_obj* gPrograms;
+vector_obj* gSaphireProgCompletions;
     // 補完で使うプログラムの名前 vector of string
 BOOL gKitutukiExit;
     // exitコマンドが実行されたらTRUE
@@ -90,6 +90,7 @@ char* gStatmentKindStrs[kCommand+1] = {
     "each",
     "foreach",
 //    "parse",
+    "inner_command",
     "command"
 };
 
@@ -124,10 +125,6 @@ list_obj* gSigChldRecieved;
 char* gMagicNumber = "SAPHIRE_OBJ";  
     // オブジェクトファイルのマジックナンバー
 
-hash_obj* gFileHandles;
-    // キー ファイル名
-    // 内容 ファイルディスクリプタ
-
 hash_obj* gReadBuffers;
     // キー ファイル名
     // 内容 sRFd*
@@ -197,6 +194,9 @@ vector_obj* gStackTraceFName;
 vector_obj* gStackTraceLineNum;
     // スタックの記録
 
+BOOL gSaphireBackground;
+    // Saphireがバックグラウンドで動いているかどうか
+
 //////////////////////////////////////////////////////////
 // 内部関数
 //////////////////////////////////////////////////////////
@@ -288,7 +288,6 @@ void string_pomch(string_obj* str)
 // closeするときにそのファイルディスクリプタに
 // gFReadBuffersのバッファがあるがあれば削除
 // gFWriteBUffersのバッファがあればバッファを書き込んで削除
-// gFileHandlesにファイルハンドルがあれば削除
 BOOL xclose(int fd) 
 {
     char buf[32];
@@ -300,11 +299,6 @@ BOOL xclose(int fd)
         hash_erase(gFReadBuffers, buf);
     }
 
-    char* key = hash_key(gFileHandles, (char*)fd);
-    if(key) {
-        hash_erase(gFileHandles, key);
-    }
-
     close(fd);
     return TRUE;
 }
@@ -363,7 +357,7 @@ void err_msg(char* msg, char* sname, int line)
 int get_command_kind(char* arg0)
 {
     int i;
-    for(i=0; i<kCommand; i++) {
+    for(i=0; i<kInnerCommand; i++) {
         if(strcmp(arg0, gStatmentKindStrs[i]) == 0) {
             return i;
         }
@@ -2410,25 +2404,25 @@ static void saphire_rehash_without_user_fun()
     hash_obj* check = HASH_NEW(3000);
 
     int i;
-    for(i=0; i<vector_size(gPrograms); i++) {
-        string_delete(vector_item(gPrograms, i));
+    for(i=0; i<vector_size(gSaphireProgCompletions); i++) {
+        string_delete(vector_item(gSaphireProgCompletions, i));
     }
-    vector_clear(gPrograms);
+    vector_clear(gSaphireProgCompletions);
 
     /// 内部コマンド ///
     for(i=0; i<kCommand; i++) {
-        vector_add(gPrograms, STRING_NEW(gStatmentKindStrs[i]));
+        vector_add(gSaphireProgCompletions, STRING_NEW(gStatmentKindStrs[i]));
     }
 
     /// 外部登録の内部コマンド ///
     hash_it* it = hash_loop_begin(gInnerCommands);
     while(it != NULL) {
         sInnerCommand* command = hash_loop_item(it);
-        vector_add(gPrograms, STRING_NEW(string_c_str(command->mName)));
+        vector_add(gSaphireProgCompletions, STRING_NEW(string_c_str(command->mName)));
         it = hash_loop_next(it);
     }
 
-    /// $PATHにあるファイルを全てgProgramsに入れていく ///
+    /// $PATHにあるファイルを全てgSaphireProgCompletionsに入れていく ///
     char* path = getenv("PATH");
     if(path == NULL) {
         fprintf(stderr, "$PATH is NULL\n");
@@ -2475,15 +2469,15 @@ static void saphire_rehash_without_user_fun()
                                 memcpy(buf, entry->d_name
                                         , strlen(entry->d_name) -4);
                                 buf[strlen(entry->d_name) -4] = 0;
-                                vector_add(gPrograms, STRING_NEW(buf));
+                                vector_add(gSaphireProgCompletions, STRING_NEW(buf));
                             }
                             else {
-                                vector_add(gPrograms
+                                vector_add(gSaphireProgCompletions
                                         , STRING_NEW(entry->d_name));
                             }
                             
 #else
-                            vector_add(gPrograms
+                            vector_add(gSaphireProgCompletions
                                         , STRING_NEW(entry->d_name));
 #endif
                         }
@@ -2507,14 +2501,14 @@ static void saphire_rehash_without_user_fun()
             if(strcmp(entry->d_name, ".") != 0
                     && strcmp(entry->d_name, "..") != 0)
             {
-                vector_add(gPrograms, STRING_NEW(entry->d_name));
+                vector_add(gSaphireProgCompletions, STRING_NEW(entry->d_name));
             }
         }
 
         closedir(dir);
     }
 
-    (void)vector_sort(gPrograms, name_sort);
+    (void)vector_sort(gSaphireProgCompletions, name_sort);
 
     hash_delete(check);
 
@@ -2526,7 +2520,7 @@ static void saphire_rehash_user_fun()
     hash_it* it = hash_loop_begin(gFuncs);
     while(it != NULL) {
         sFunction* fun = hash_loop_item(it);
-        vector_add(gPrograms, STRING_NEW(string_c_str(fun->name)));
+        vector_add(gSaphireProgCompletions, STRING_NEW(string_c_str(fun->name)));
         it = hash_loop_next(it);
     }
 }
@@ -2667,10 +2661,9 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gGlobals = HASH_NEW(100);
 
-    gFileHandles = HASH_NEW(100);
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "1.0.8", 1);
+    setenv("SAPHIRE_VERSION", "1.1.1", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2687,7 +2680,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
         setenv("SAPHIRE_USERRC", path, 1);
     }
     
-    gPrograms = VECTOR_NEW(100);
+    gSaphireProgCompletions = VECTOR_NEW(100);
 
     gRegexs = HASH_NEW(100);
     gRegexsI = HASH_NEW(100);
@@ -2729,7 +2722,14 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
             read_rc_obj_file(run_user_runtimescript);
             break;
     }
-    mreset_tty();
+    if(tcgetpgrp(0) == getpgid(0)) {
+        gSaphireBackground = FALSE;
+        mreset_tty();
+    }
+    else {
+        gSaphireBackground = TRUE;
+    }
+
     char* env = getenv("SAPHIRE_TMPDIR");
     if(env) 
         strcpy(gTmpDir, env);
@@ -2741,6 +2741,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gStackTraceLineNum = VECTOR_NEW(30);
 
     tmpfile_all_sweep();
+    gSaphireVarCompletions = VECTOR_NEW(100);
 }
 
 void saphire_final()
@@ -2833,14 +2834,6 @@ void saphire_final()
 
     list_delete(gSigChldRecieved);
 
-    it = hash_loop_begin(gFileHandles);
-    while(it) {
-        int fd = (int)hash_loop_item(it);
-        if(fd != 0 && fd != 1 && fd != 2) (void)close(fd);
-        it = hash_loop_next(it);
-    }
-    hash_delete(gFileHandles);
-
     it = hash_loop_begin(gReadBuffers);
     while(it) {
         sRFd* fd = (sRFd*)hash_loop_item(it);
@@ -2850,10 +2843,10 @@ void saphire_final()
     }
     hash_delete(gReadBuffers);
     
-    for(i=0; i<vector_size(gPrograms); i++) {
-        string_delete(vector_item(gPrograms, i));
+    for(i=0; i<vector_size(gSaphireProgCompletions); i++) {
+        string_delete(vector_item(gSaphireProgCompletions, i));
     }
-    vector_delete(gPrograms);
+    vector_delete(gSaphireProgCompletions);
 
     it = hash_loop_begin(gRegexs);
     while(it != NULL) {
@@ -2936,6 +2929,11 @@ void saphire_final()
 
     string_delete(gAncestalMemoryPipeIn);
     string_delete(gAncestalMemoryPipeOut);
+
+    for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
+        string_delete(vector_item(gSaphireVarCompletions, i));
+    }
+    vector_delete(gSaphireVarCompletions);
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -3620,3 +3618,76 @@ BOOL sWFd_flash(sWFd* self)
 
     return TRUE;
 }
+
+vector_obj* gSaphireVarCompletions;               // 環境変数補完候補
+
+void saphire_make_var_completion_list()
+{
+    int i;
+    for(i=0; i<vector_size(gSaphireVarCompletions); i++) {
+        string_delete(vector_item(gSaphireVarCompletions, i));
+    }
+    vector_clear(gSaphireVarCompletions);
+
+    /// 環境変数 ///
+#if HAVE_DECL_ENVIRON
+    extern char **environ;
+
+    char** p;
+    for(p = environ; *p; p++) {
+        char env_name[1024];
+
+        char* p2 = env_name;
+        char* p3 = *p;
+
+        while(*p3 != '=') {
+            p3++;
+        }
+
+        if(p3 - *p < 1024) {
+            memcpy(env_name, *p, p3 - *p);
+            env_name[p3 - *p] = 0;
+        }
+
+        vector_add(gSaphireVarCompletions, STRING_NEW(env_name));
+    }
+#endif
+
+    hash_it* it = hash_loop_begin(gGlobals);
+    while(it) {
+        char* key = hash_loop_key(it);
+
+        char env_name[1024];
+        strncpy(env_name, key, 1024);
+
+        vector_add(gSaphireVarCompletions, STRING_NEW(env_name));
+
+        it = hash_loop_next(it);
+    }
+
+    it = hash_loop_begin(gArrays);
+    while(it) {
+        char* key = hash_loop_key(it);
+
+        char env_name[1024];
+        strncpy(env_name, key, 1024);
+
+        vector_add(gSaphireVarCompletions, STRING_NEW(env_name));
+
+        it = hash_loop_next(it);
+    }
+
+    it = hash_loop_begin(gHashs);
+    while(it) {
+        char* key = hash_loop_key(it);
+
+        char env_name[1024];
+        strncpy(env_name, key, 1024);
+
+        vector_add(gSaphireVarCompletions, STRING_NEW(env_name));
+
+        it = hash_loop_next(it);
+    }
+
+    (void)vector_sort(gSaphireVarCompletions, name_sort);
+}
